/*
   This program parses and then evaluates strings from Language-9.
*/

public class Language_9_Examples
{
   public static void main(String[] args)
   {
      // IMPORTANT: Set this to 0 or 1 depending on whether you need
      // to see all of the interpreter's debugging information.
      Evaluate.DEBUG = 1;

      String[] programs =
      {
         "(prog" +
            "(var x1 10)" +
            "(var x2 20)" +
            "(var x3 (list x1 100 true x2 (list 1 2 3)))" +
            "(var r1 (empty (cdr x3)))" +
            "(var r2 (car x3))" +
            "(var r3 (cons true (cdr x3))))",

         "(prog (cons 5 (cdr (cdr (list 1 2)))))",

         "(prog (cons 1 (cons (list 2 3 4 5) (cons 6 (list 7 8 9)))))",

         "(prog (list 1 (list 2 3 4 5)))",

         "(prog (cons 1 (list 2 3 4 5)))",

         "(prog" +
           "(fun map" +
               "(lambda op lst" +  // op is a function, lst is a list
                 "(if (empty lst)" +
                      "lst" +
                      "(cons (apply op (car lst)) (apply map op (cdr lst))))))" +
           "(var r1 (apply map (lambda x (+ x 10)) (list 1 2 3 4 5 6 7)))" +
           "(var r2 (apply map (lambda x (* x 10)) (list 1 2 3 4 5 6 7))))",

         "(prog" +
           "(fun reduce" +
               "(lambda op lst" +  // op is a function, lst is a non-empty list
                 "(if (empty (cdr lst))" +
                      "(car lst)" +
                      "(apply op (car lst) (apply reduce op (cdr lst))))))" +
           "(var r1 (apply reduce (lambda x y (+ x y)) (list 1 2 3 4 5 6 7)))" +
           "(var r2 (apply reduce (lambda x y (* x y)) (list 1 2 3 4 5 6 7))))",

         "(prog\n" +
           "(fun map\n" +
               "(lambda op lst\n" +  // op is a function, lst is a list
                 "(if (empty lst)\n" +
                      "lst\n" +
                      "(cons (apply op (car lst)) (apply map op (cdr lst))))))\n" +
           "(fun reduce\n" +
               "(lambda op lst\n" +  // op is a function, lst is a non-empty list
                 "(if (empty (cdr lst))\n" +
                      "(car lst)\n" +
                      "(apply op (car lst) (apply reduce op (cdr lst))))))\n" +
           "(var nums (list 1 2 3 4 5 6 7))\n" +
           "(var result (apply reduce (lambda x y (+ x y)) (apply map (lambda x (+ x 10)) nums))))\n"
      };


      for (int i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =========================================");

         // Build and evaluate the AST that represents the expression.
         try
         {
            final Tree ast = ParseTree.buildTree( programs[i] );

            // Print the AST as an S-expression
            //System.out.println( ast + "\n" );

            // Pretty-print the abstract syntax tree.
            //System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Evaluate the expression (interpret the AST).
            try
            {
               final Value value = Evaluate.eval( ast );
               if (Evaluate.DEBUG > 0)
               {
                  System.out.println("result = " + value + "\n" );
               }
               else
               {
                  System.out.println("result = " + value.toSimpleString() + "\n" );
               }
            }
            catch (EvalException e)
            {
               System.out.println(e);
             //e.printStackTrace();
            }


            // Create dot and png files from the AST.
            if (Evaluate.DEBUG > 0)
            try
            {
               // Create the (empty) dot file.
               final String baseName = String.format("Language_9.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               final String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                                     "-Tpng",
                                     baseName + ".dot",
                                     "-o",
                                     baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }
         }
         catch (TokenizeException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }
}
