/**
   This class is a kind of "tagged union". Each instance of this class holds
   a value for our simple language. But our language has three kinds of values,
   ints, booleans, and functions. So each instance of this class holds both an
   int value, a boolean value, and a Tree value. The "tag" field is used to
   determine which of the three values that an instance holds, the int, boolean,
   or Tree, is valid.

   See also
      http://en.wikipedia.org/wiki/Tagged_union
*/

public class Value
{
   public String tag;     // either "int" or "bool" or "lambda" or "list"
   public int valueI = 0;
   public boolean valueB = false;
   public IPEP valueL = null; // changed valueL from Tree to IPEP (Tree and Environment)
   public ConsCell valueCC = null;

   public static final String INT_TAG = "int";
   public static final String BOOL_TAG = "bool";
   public static final String LAMBDA_TAG = "lambda";
   public static final String LIST_TAG = "list";

   /**
      Construct a Value object that holds an int value.
   */
   public Value(int value)
   {
      this.tag = INT_TAG;
      this.valueI = value;
   }

   /**
      Construct a Value object that holds a boolean value.
   */
   public Value(boolean value)
   {
      this.tag = BOOL_TAG;
      this.valueB = value;
   }

   /**
      Construct a Value object that holds a "closure".
   */
   public Value(IPEP value)
   {
      this.tag = LAMBDA_TAG;
      this.valueL = value;
   }

   /**
      Construct a Value object that holds a list.
   */
   public Value(ConsCell value)
   {
      this.tag = LIST_TAG;
      this.valueCC = value;
   }

   /**
      Construct a Value object that holds an empty list value.
   */
   public Value()
   {
      this.tag = LIST_TAG;
      this.valueCC = null;
   }
/*
   public String toString()
   {
      return "[tag->" + tag
                      + ", valueI->" + valueI
                      + ", valueB->" + valueB
                      + ", valueL->" + valueL
                      + ", valueCC->" + valueCC
                      + "]";
   }
*/
   public String toSimpleString()
   {
      return toString();
   }

   public String toString()
   {
      String result = "";

      if ( tag.equals(BOOL_TAG) )
      {
         result += valueB;
      }
      else if ( tag.equals(INT_TAG) )
      {
         result += valueI;
      }
      else if ( tag.equals(LAMBDA_TAG) )
      {
         result += valueL;
      }
      else if ( tag.equals(LIST_TAG) )
      {
         result += "[" + valueCC + " ]";
      }
      else  // bad tag (shouldn't get here)
      {
         result += "[tag->" + tag
                            + ", valueI->" + valueI
                            + ", valueB->" + valueB
                            + ", valueL->" + valueL
                            + ", valueCC->" + valueCC
                            + "]";
      }

      return result;
   }
}//Value
